//
//  AliCloudMNSiOSTests.m
//  AliCloudMNSiOSTests
//
//  Created by David Chen on 28/03/2017.
//  Copyright © 2017 AliCloud. All rights reserved.
//

#import <XCTest/XCTest.h>
#import "AliCloudMNSiOS.h"

/*Set the endpoint, access ak  before run test.*/
NSString* G_Config_Endpoint = @"http://*****.mns.cn-hangzhou.aliyuncs.com";
NSString* G_Config_AccessId = @"*****";
NSString* G_Config_AccessKey = @"*****";

@interface AliCloudMNSiOSTests : XCTestCase
@property AliMNSAccount* account;
@property AliMNSAccount* accountWithWrongAK;
@property NSString* testQueueName1; // for testCreateQueue
@property NSString* testQueueName2; // for testDeleteQueue
@property NSString* testQueueName3; // for other queue operations.

@property NSString* testTopicName1;
@property NSString* testTopicName2;
@property NSString* testTopicName3;

@property NSString* testSubName1;
@property NSString* testSubName2;

@property NSString* invalidName;
@property NSInteger visibilityTimeout;

@property AliMNSQueueMeta* queueMeta1;
@property AliMNSQueueMeta* queueMeta2;
@property AliMNSQueueMeta* queueMeta3;

@property AliMNSTopicMeta* topicMeta1;
@property AliMNSTopicMeta* topicMeta2;
@property AliMNSTopicMeta* topicMeta3;

@end

@implementation AliCloudMNSiOSTests

@synthesize account;
@synthesize accountWithWrongAK;
@synthesize testQueueName1;
@synthesize testQueueName2;
@synthesize testQueueName3;
@synthesize testTopicName1;
@synthesize testTopicName2;
@synthesize testTopicName3;
@synthesize invalidName;
@synthesize visibilityTimeout;
@synthesize queueMeta1;
@synthesize queueMeta2;
@synthesize queueMeta3;

@synthesize topicMeta1;
@synthesize topicMeta2;
@synthesize topicMeta3;

@synthesize testSubName1;
@synthesize testSubName2;

- (void)setUp {
    [super setUp];
    if (account == nil)
    {
        account = [[AliMNSAccount alloc]
                    initWithEndpoint:G_Config_Endpoint withAccessId:G_Config_AccessId withAccessKey:G_Config_AccessKey];

        accountWithWrongAK = [[AliMNSAccount alloc]
                              initWithEndpoint:G_Config_Endpoint
                              withAccessId:G_Config_AccessId
                              withAccessKey:@""];
        testQueueName1 = @"myiossdk-testqueue1";
        testQueueName2 = @"myiossdk10-testqueue2";
        testQueueName3 = @"myiossdk10-testqueue3";
        testTopicName1 = @"myios-test-topic-1";
        testTopicName2 = @"myios-test-topic-2";
        testTopicName3 = @"myios-test-topic-3";
        testSubName1 = @"myiossdk-testsub-1";
        testSubName2 = @"myiossdk-testsub-2";
        invalidName = @"myiossdk_invalidName";
        visibilityTimeout = 5;
    }

    queueMeta1 = [[AliMNSQueueMeta alloc] initWithQueueName:testQueueName1];
    queueMeta1.delaySeconds = 10;
    queueMeta1.visibilityTimeout = visibilityTimeout;
    queueMeta1.messageRetentionPeriod = 86400;
    queueMeta1.maximumMessageSize = 65530;

    // prepare for delete queue test case
    queueMeta2 = [[AliMNSQueueMeta alloc] initWithQueueName:testQueueName2];
    queueMeta2.delaySeconds = 10;
    queueMeta2.visibilityTimeout = visibilityTimeout;
    queueMeta2.messageRetentionPeriod = 86400;
    queueMeta2.maximumMessageSize = 65530;
    AliMNSQueue* queue2 = [account getQueue:testQueueName2];
    [queue2 create:queueMeta2];

    queueMeta3 = [[AliMNSQueueMeta alloc] initWithQueueName:testQueueName3];
    queueMeta3.visibilityTimeout = visibilityTimeout;
    queueMeta3.messageRetentionPeriod = 86400;
    queueMeta3.maximumMessageSize = 65530;

    // prepare queue3 for other operations test casse.
    AliMNSQueue* queue3 = [account getQueue:testQueueName3];
    [queue3 create:queueMeta3];

    // send 8 messages for test.
    for(int i = 0;  i < 8; i++)
    {
        AliMNSMessage* message = [AliMNSMessage new];
        message.messageBody = @"test message";

        [queue3 sendMessage:message];
    }

    // prepare for topics
    topicMeta1 = [[AliMNSTopicMeta alloc] initWithTopicName:testTopicName1];
    topicMeta1.maximumMessageSize = 65535;

    topicMeta2 = [[AliMNSTopicMeta alloc] initWithTopicName:testTopicName2];
    topicMeta2.maximumMessageSize = 65535;
    AliMNSTopic* topic2 = [account getTopic:testTopicName2];
    [topic2 create:topicMeta2];

    topicMeta3 = [[AliMNSTopicMeta alloc] initWithTopicName:testTopicName3];
    topicMeta3.maximumMessageSize = 65535;
    AliMNSTopic* topic3 = [account getTopic:testTopicName3];
    [topic3 create:topicMeta3];
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
    AliMNSQueue* queue1 = [account getQueue:testQueueName1];
    [queue1 deleteQueue];

    AliMNSQueue* queue2 = [account getQueue:testQueueName2];
    [queue2 deleteQueue];

    AliMNSQueue* queue3 = [account getQueue:testQueueName3];
    [queue3 deleteQueue];

    AliMNSTopic* topic1 = [account getTopic:testTopicName1];
    [topic1 deleteTopic];

    AliMNSTopic* topic2 = [account getTopic:testTopicName2];
    [topic2 deleteTopic];

    AliMNSTopic* topic3 = [account getTopic:testTopicName3];
    [topic3 deleteTopic];

    [super tearDown];
}

/*test cases for Queue apis*/

- (void) testCreateQueueNormal {
    AliMNSQueue* queue = [account getQueue:testQueueName1];
    NSLog(@"Begin: create queue:\n");
    @try
    {
        NSString* queueUrl = [queue create:queueMeta1];
        XCTAssert(queueUrl != nil, @"create queue fail\n");
        NSLog(@"End:create queue successfully: queueUrl=%@\n", queueUrl);
    }@catch(AliMNSClientNetworkException* cne)
    {
        XCTAssert(false, @"create queue fail\n");
        NSLog(@"AliMNSClientNetworkException: errorStr=%@, subMessage=%@\n",  [cne.error localizedDescription], cne.subMessage);
    }@catch(AliMNSServerException* se)
    {
        XCTAssert(false, @"create queue fail\n");
        NSLog(@"AliMNSServerException: errorCode=%@, requestId=%@, message=%@, hostId=%@", se.errorCode, se.requestId, se.message, se.hostId);
    }
}

- (void) testCreateQueueAbnormal {
    AliMNSQueue* queue = [account getQueue:invalidName];
    AliMNSQueueMeta* qMeta = [[AliMNSQueueMeta alloc] initWithQueueName:invalidName];
    qMeta.delaySeconds = 10;
    qMeta.visibilityTimeout = 50;
    qMeta.messageRetentionPeriod = 86400;
    qMeta.maximumMessageSize = 65530;
    NSLog(@"Begin: create queue:\n");
    @try
    {
        [queue create:qMeta];
        XCTAssert(false, @"testCreateQueueAbnormal fail, expect create queue fail.\n");
    }@catch(AliMNSClientNetworkException* cne)
    {
        XCTAssert(false, @"Exception type is wrong.\n");
    }@catch(AliMNSServerException* se)
    {
        XCTAssert(true, @"create queue fail\n");
        XCTAssert([se.errorCode isEqual:@"InvalidQueueName"], "errorCode should be InvalidQueueName");
        NSLog(@"AliMNSServerException: errorCode=%@, requestId=%@, message=%@, hostId=%@", se.errorCode, se.requestId, se.message, se.hostId);
    }
}

- (void) testDeleteQueue {

    AliMNSQueue* queue = [account getQueue:testQueueName2];
    @try {
        [queue deleteQueue];
        XCTAssert(true, @"delete queue failed\n");
    } @catch (AliMNSClientNetworkException* cne) {
        NSLog(@"AliMNSClientNetworkException: errorStr=%@, subMessage=%@\n",  [cne.error localizedDescription], cne.subMessage);
        XCTAssert(false, @"delete queue failed due network issue \n");
    }@catch (AliMNSServerException* se)
    {
        NSLog(@"AliMNSServerException: errorCode=%@, requestId=%@, message=%@, hostId=%@", se.errorCode, se.requestId, se.message, se.hostId);
        XCTAssert(false, @"delete queue due to server side error\n");
    }
}

/* queue not exist*/
- (void) testDeleteQueueAbnormal {
    AliMNSQueue* queue = [account getQueue:@"not_exist-queue"];
    @try{
        [queue deleteQueue];
        XCTAssert(false, @"expected delete queue fail.\n");
    } @catch (AliMNSClientNetworkException* cne)
    {
        NSLog(@"AliMNSClientNetworkException: errorStr=%@, subMessage=%@\n",  [cne.error localizedDescription], cne.subMessage);
        XCTAssert(false, @"delete queue failed due network issue \n");
    } @catch (AliMNSServerException* se)
    {
        NSLog(@"AliMNSServerException: errorCode=%@, requestId=%@, message=%@, hostId=%@", se.errorCode, se.requestId, se.message, se.hostId);
        XCTAssert([se.errorCode isEqual:@"InvalidQueueName"], @"expected error code:InvalidQueueName\n");
    }
}

- (void) testListQueue
{
    @try{
        NSString* nextMaker = [NSString new];
        NSArray* qUrlList = [account listQueue: @"" withReturnNumber:10 withMarker:&nextMaker];
        XCTAssert(qUrlList.count > 0, @"expected list queue successfully.\n");
    }@catch (AliMNSClientNetworkException* cne)
    {
        NSLog(@"AliMNSClientNetworkException: errorStr=%@, subMessage=%@\n",  [cne.error localizedDescription], cne.subMessage);
        XCTAssert(false, @"delete queue failed due network issue \n");

    }@catch (AliMNSServerException* se)
    {
        NSLog(@"AliMNSServerException: errorCode=%@, requestId=%@, message=%@, hostId=%@", se.errorCode, se.requestId, se.message, se.hostId);
        XCTAssert(false, @"delete queue due to server side error\n");
    }
}

-(void) testGetQeueueAttributes
{
    AliMNSQueue* queue = [account getQueue:testQueueName2];
    AliMNSQueueMeta* getMeta = [queue getAttributes];

    XCTAssert(queueMeta2.delaySeconds == getMeta.delaySeconds, @"expected delaySeconds is correct");
    XCTAssert(queueMeta2.visibilityTimeout == getMeta.visibilityTimeout, @"expected delaySeconds is correct");
    XCTAssert(queueMeta2.messageRetentionPeriod == getMeta.messageRetentionPeriod, @"expected delaySeconds is correct");
    XCTAssert(queueMeta2.maximumMessageSize == getMeta.maximumMessageSize, @"expected delaySeconds is correct");
}

-(void) testSetQeueueAttributes
{
    AliMNSQueue* queue = [account getQueue:testQueueName3];
    queueMeta3.delaySeconds = 20;
    [queue setAttributes: queueMeta3];

    AliMNSQueueMeta* getMeta = [queue getAttributes];

    XCTAssert(queueMeta3.delaySeconds == getMeta.delaySeconds, @"expected delaySeconds is correct");
}

- (void) testSendMessage
{
    AliMNSQueue* queue = [account getQueue:testQueueName3];
    AliMNSMessage* message = [AliMNSMessage new];
    message.messageBody = @"test message";

    AliMNSMessage* retMsg = [queue sendMessage:message];

    NSLog(@"reqId=%@, msgId=%@, msgMd5=%@, msgHandler=%@", retMsg.requestId, retMsg.messageId, retMsg.messageBodyMd5, retMsg.receiptHandle);

    XCTAssert(retMsg != nil, @"return message should not be nil.");
    XCTAssert(retMsg.messageId != nil, @"message id should not be nil");
    XCTAssert(retMsg.messageBodyMd5 != nil, @"message body md5 should not be nil");
}

- (void) testBatchSendMessage
{
    AliMNSQueue* queue = [account getQueue:testQueueName3];
    NSUInteger numOfMsg = 16;
    NSMutableArray* msgList = [NSMutableArray new];

    for (int i= 0; i < numOfMsg; i++)
    {
        AliMNSMessage* message = [AliMNSMessage new];
        message.messageBody = [NSString stringWithFormat:@"test message %ld", (long)i];
        [msgList addObject:message];
    }

    NSArray* toSendMsgList = [msgList copy];

    NSArray* sendResultList = [queue batchSendMessage:toSendMsgList];

    XCTAssert(sendResultList.count == numOfMsg, @"message number is incorrect.\n");
    for (int i = 0; i < sendResultList.count; i++)
    {
        AliMNSMessage* retMsg = sendResultList[i];
        NSLog(@"reqId=%@, msgId=%@, msgMd5=%@, msgHandler=%@\n", retMsg.requestId, retMsg.messageId, retMsg.messageBodyMd5, retMsg.receiptHandle);
        XCTAssert(retMsg != nil, @"return message should not be nil.");
        XCTAssert(retMsg.messageId != nil, @"message id should not be nil");
        XCTAssert(retMsg.messageBodyMd5 != nil, @"message body md5 should not be nil");
    }
}

- (void) testPeekMessage
{
    AliMNSQueue* queue = [account getQueue:testQueueName3];
    AliMNSMessage* msg = [queue peekMessage];

    XCTAssert(msg != nil, @"return message should not be nil.");
    XCTAssert(msg.messageId != nil, @"message id should not be nil");
    XCTAssert(msg.messageBodyMd5 != nil, @"message body md5 should not be nil");
    XCTAssert(msg.messageBody != nil, @"message body should not be nil");
    XCTAssert(msg.dequeueCount >= 0, @"message deqeueu count should bigger than 1");
}

- (void) testBatchPeekMessage
{
    NSUInteger numOfMsg = 3;
    AliMNSQueue* queue = [account getQueue:testQueueName3];
    NSArray* msgList = [queue batchPeekMessage:numOfMsg];

    XCTAssert(msgList.count == numOfMsg, @"message number is incorrect.\n");
    for(int i = 0; i < msgList.count; i++)
    {
        AliMNSMessage* msg = msgList[i];
        XCTAssert(msg != nil, @"return message should not be nil.");
        XCTAssert(msg.messageId != nil, @"message id should not be nil");
        XCTAssert(msg.messageBodyMd5 != nil, @"message body md5 should not be nil");
        XCTAssert(msg.messageBody != nil, @"message body should not be nil");
        XCTAssert(msg.dequeueCount >= 0, @"message deqeueu count should bigger than 1");
    }
}

-(void) testReceiveMessage
{
    AliMNSQueue* queue = [account getQueue:testQueueName3];
    AliMNSMessage* msg = [queue receiveMessage:0];

    XCTAssert(msg != nil, @"return message should not be nil.");
    XCTAssert(msg.messageId != nil, @"message id should not be nil");
    XCTAssert(msg.messageBodyMd5 != nil, @"message body md5 should not be nil");
    XCTAssert(msg.receiptHandle != nil, @"message receiptHandle should not be nil");
    XCTAssert(msg.messageBody != nil, @"message body should not be nil");
    XCTAssert(msg.dequeueCount >= 1, @"message deqeueu count should bigger than 1");
    XCTAssert(msg.nextVisibleTime > 0, @"message nextVisibleTime should bigger than 0");
}

-(void) testBatchReceiveMessage
{
    NSUInteger numOfMsg = 3;
    AliMNSQueue* queue = [account getQueue:testQueueName3];
    NSArray* msgList = [queue batchReceiveMessage:2 withBatchSize: numOfMsg];

    XCTAssert(msgList.count == numOfMsg, @"message number is incorrect.\n");
    for(int i = 0; i < msgList.count; i++)
    {
        AliMNSMessage* msg = msgList[i];
        XCTAssert(msg != nil, @"return message should not be nil.");
        XCTAssert(msg.messageId != nil, @"message id should not be nil");
        XCTAssert(msg.messageBodyMd5 != nil, @"message body md5 should not be nil");
        XCTAssert(msg.receiptHandle != nil, @"message receiptHandle should not be nil");
        XCTAssert(msg.messageBody != nil, @"message body should not be nil");
        XCTAssert(msg.dequeueCount >= 1, @"message deqeueu count should bigger than 1");
        XCTAssert(msg.nextVisibleTime > 0, @"message nextVisibleTime should bigger than 0");
    }
}

-(void) testDeleteMessasge
{
    AliMNSQueue* queue = [account getQueue:testQueueName3];
    AliMNSMessage* msg = [queue receiveMessage:0];

    XCTAssert(msg != nil, @"return message should not be nil.");
    XCTAssert(msg.messageId != nil, @"message id should not be nil");
    XCTAssert(msg.messageBodyMd5 != nil, @"message body md5 should not be nil");
    XCTAssert(msg.receiptHandle != nil, @"message receiptHandle should not be nil");
    XCTAssert(msg.messageBody != nil, @"message body should not be nil");
    XCTAssert(msg.dequeueCount >= 1, @"message deqeueu count should bigger than 1");
    XCTAssert(msg.nextVisibleTime > 0, @"message nextVisibleTime should bigger than 0");

    [queue deleteMessage:msg.receiptHandle];
    XCTAssert(true, @"no exception means delete messsage successfully.");
}

-(void) testBatchDeleteMessage
{
    NSUInteger numOfMsg = 3;
    AliMNSQueue* queue = [account getQueue:testQueueName3];
    NSArray* msgList = [queue batchReceiveMessage:2 withBatchSize: numOfMsg];
    NSMutableArray* tmpHanldeList = [NSMutableArray new];

    XCTAssert(msgList.count == numOfMsg, @"message number is incorrect.\n");
    for(int i = 0; i < msgList.count; i++)
    {
        AliMNSMessage* msg = msgList[i];
        [tmpHanldeList addObject:msg.receiptHandle];
        XCTAssert(msg != nil, @"return message should not be nil.");
        XCTAssert(msg.messageId != nil, @"message id should not be nil");
        XCTAssert(msg.messageBodyMd5 != nil, @"message body md5 should not be nil");
        XCTAssert(msg.receiptHandle != nil, @"message receiptHandle should not be nil");
        XCTAssert(msg.messageBody != nil, @"message body should not be nil");
        XCTAssert(msg.dequeueCount >= 1, @"message deqeueu count should bigger than 1");
        XCTAssert(msg.nextVisibleTime > 0, @"message nextVisibleTime should bigger than 0");
    }

    // delete the 3 messages.
    NSArray* rHanldeList = [tmpHanldeList copy];
    [queue batchDeleteMessage:rHanldeList];
    XCTAssert(true, @"no exception means delete messsage successfully.");
}

-(void) testChangeMessageVisibility
{
    AliMNSQueue* queue = [account getQueue:testQueueName3];
    AliMNSMessage* msg = [queue receiveMessage:0];

    XCTAssert(msg != nil, @"message should not be nil.");
    NSInteger nextTime = 100;

    AliMNSMessage* newMsg = [queue changeMessageVisibility:msg.receiptHandle withVisibilityTimeout: nextTime];
    XCTAssert(newMsg != nil, @"return message should not be nil.");
    XCTAssert(newMsg.receiptHandle != nil, @"message receiptHandle should not be nil");
    NSInteger delta = (newMsg.nextVisibleTime - msg.nextVisibleTime)/1000;
    XCTAssert((delta + visibilityTimeout) == nextTime, @"nextVisibleTime should be added 100s");
}

/*test cases for Topic apis*/

-(void) testCreateTopic
{
    AliMNSTopic* topic = [account getTopic:testTopicName1];
    [topic create:topicMeta1];

    XCTAssert(true, @"no exception means create topic successfully.");
}

-(void) testCreateTopicAbnormal
{
    AliMNSTopic* topic = [account getTopic:invalidName];
    AliMNSTopicMeta* tMeta = [[AliMNSTopicMeta alloc] initWithTopicName:invalidName];
    tMeta.maximumMessageSize = 5000;
    @try {
        [topic create:tMeta];
        XCTAssert(false, @"expect throw exception for invalid name.");
    } @catch (AliMNSClientNetworkException* cne) {
        XCTAssert(false, @"Exception type is wrong.\n");
    } @catch (AliMNSServerException* se)
    {
        XCTAssert(true, @"create topic fail\n");
        XCTAssert([se.errorCode isEqual:@"TopicNameInvalid"], "errorCode should be TopicNameInvalid");
        NSLog(@"AliMNSServerException: errorCode=%@, requestId=%@, message=%@, hostId=%@", se.errorCode, se.requestId, se.message, se.hostId);
    }
}

-(void) testDeleteTopic
{
    AliMNSTopic* topic = [account getTopic:testTopicName2];
    [topic deleteTopic];
    XCTAssert(true, @"no exception means delete topic successfully.");
}

-(void) testListTopic
{
    NSString* nextMaker = [NSString new];
    NSArray* tUrlList = [account listTopic:@"" withReturnNumber:10 withMarker:&nextMaker];
    XCTAssert(tUrlList.count > 0, @"expected list topic successfully.\n");
}

-(void) testSetTopicAttributes
{
    AliMNSTopic* topic = [account getTopic:testTopicName3];
    topicMeta3.maximumMessageSize = 2048;
    [topic setTopicAttributes:topicMeta3];

    AliMNSTopicMeta* newTopicMeta = [topic getTopicAttributes];
    XCTAssert((topicMeta3.maximumMessageSize == newTopicMeta.maximumMessageSize), @"expected set topic attibutes successfully.\n");
}

-(void) testGetTopicAttributes
{
    AliMNSTopic* topic = [account getTopic:testTopicName3];

    AliMNSTopicMeta* newTopicMeta = [topic getTopicAttributes];
    XCTAssert((topicMeta3.maximumMessageSize == newTopicMeta.maximumMessageSize), @"expected get topic attibutes successfully.\n");
}

-(void) testPublishMessage
{
    AliMNSTopic* topic = [account getTopic:testTopicName3];
    AliMNSTopicMessage* tMsg = [AliMNSTopicMessage new];
    tMsg.messageBody = @"test topic message";
    tMsg.messageTag = @"Ali";

    AliMNSTopicMessage* retMsg = [topic publishMessage:tMsg];
    XCTAssert(retMsg != nil, @"return message should not be nil.");
    XCTAssert(retMsg.messageId != nil, @"message id should not be nil");
    XCTAssert(retMsg.messageBodyMd5 != nil, @"message body md5 should not be nil");
}

-(void) testSubscribe
{
    AliMNSTopic* topic = [account getTopic:testTopicName3];

    AliMNSSubscription* sub = [[AliMNSSubscription alloc] initWithTopicName: testTopicName3 withSubscriptionName:testSubName1];

    sub.filterTag = @"testFlag";
    sub.endpoint = @"https://test1.com";
    sub.notifyStrategy = ALIMNS_TOPIC_BACKOFF_RETRY;
    sub.notifyContentFormat = ALIMNS_TOPIC_SIMPLIFIED;

    NSString* suburl = [topic subscribe:sub];
    XCTAssert(suburl != nil, @"expected subscribe successfully.\n");
}

-(void) testSubscribeAbnormal
{
    AliMNSTopic* topic = [account getTopic:testTopicName3];
    @try {
        AliMNSSubscription* sub = [[AliMNSSubscription alloc] initWithTopicName: testTopicName3 withSubscriptionName:invalidName];

        sub.filterTag = @"testFlag";
        sub.endpoint = @"https://test1.com";
        sub.notifyStrategy = ALIMNS_TOPIC_BACKOFF_RETRY;
        sub.notifyContentFormat = ALIMNS_TOPIC_SIMPLIFIED;
        [topic subscribe:sub];
        XCTAssert(false, @"subscribe should fail\n");
    } @catch (AliMNSClientNetworkException* cne) {
        XCTAssert(false, @"Exception type is wrong.\n");
    } @catch (AliMNSServerException* se)
    {
        XCTAssert(true, @"subscribe failed\n");
        XCTAssert([se.errorCode isEqual:@"SubscriptionNameInvalid"], "errorCode should be SubscriptionNameInvalid");
        NSLog(@"AliMNSServerException: errorCode=%@, requestId=%@, message=%@, hostId=%@", se.errorCode, se.requestId, se.message, se.hostId);
    }
}

-(void) testUnsubscribe
{
    AliMNSTopic* topic = [account getTopic:testTopicName3];

    AliMNSSubscription* sub = [[AliMNSSubscription alloc] initWithTopicName: testTopicName3 withSubscriptionName:testSubName1];

    sub.filterTag = @"testFlag";
    sub.endpoint = @"https://test1.com";
    sub.notifyStrategy = ALIMNS_TOPIC_BACKOFF_RETRY;
    sub.notifyContentFormat = ALIMNS_TOPIC_SIMPLIFIED;

    NSString* suburl = [topic subscribe:sub];
    XCTAssert(suburl != nil, @"expected subscribe successfully.\n");

    [topic unsubscribe:sub];
    XCTAssert(true, @"expected subscribe successfully.\n");
}

-(void) testUnsubscribeAbnormal
{
    AliMNSTopic* topic = [account getTopic:testTopicName3];
    @try {
        AliMNSSubscription* sub = [[AliMNSSubscription alloc] initWithTopicName: testTopicName3 withSubscriptionName:invalidName];

        [topic unsubscribe:sub];
        XCTAssert(false, @"unsubscribe should fail\n");
    } @catch (AliMNSClientNetworkException* cne) {
        XCTAssert(false, @"Exception type is wrong.\n");
    } @catch (AliMNSServerException* se)
    {
        XCTAssert(true, @"unsubscribe failed\n");
        XCTAssert([se.errorCode isEqual:@"SubscriptionNameInvalid"], "errorCode should be SubscriptionNameInvalid");
        NSLog(@"AliMNSServerException: errorCode=%@, requestId=%@, message=%@, hostId=%@", se.errorCode, se.requestId, se.message, se.hostId);
    }
}

-(void) testListSubscription
{
    NSString* nextMaker = [NSString new];
    AliMNSTopic* topic = [account getTopic:testTopicName3];

    for(NSInteger i = 0; i < 4; i++)
    {
        NSString* subName = [NSString stringWithFormat:@"autoSubName-%ld", i];
        AliMNSSubscription* sub = [[AliMNSSubscription alloc] initWithTopicName: testTopicName3 withSubscriptionName:subName];

        sub.filterTag = @"testFlag";
        sub.endpoint = @"https://test2.com";
        sub.notifyStrategy = ALIMNS_TOPIC_BACKOFF_RETRY;
        sub.notifyContentFormat = ALIMNS_TOPIC_SIMPLIFIED;

        NSString* suburl = [topic subscribe:sub];
        XCTAssert(suburl != nil, @"expected subscribe successfully.\n");
    }

    NSArray* sUrlList = [topic listSubscription:@"" withReturnNumber:2 withMarker:&nextMaker];
    XCTAssert(sUrlList.count == 2, @"expected list topic successfully.\n");
    XCTAssert(![nextMaker isEqual: @""], @"expected list topic successfully.\n");
}

-(void) testSetSubscriptionAttributes
{
    AliMNSTopic* topic = [account getTopic:testTopicName3];

    AliMNSSubscription* sub = [[AliMNSSubscription alloc] initWithTopicName: testTopicName3 withSubscriptionName:testSubName2];

    sub.filterTag = @"testFlag";
    sub.endpoint = @"https://test2.com";
    sub.notifyStrategy = ALIMNS_TOPIC_BACKOFF_RETRY;
    sub.notifyContentFormat = ALIMNS_TOPIC_SIMPLIFIED;

    NSString* suburl = [topic subscribe:sub];
    XCTAssert(suburl != nil, @"expected subscribe successfully.\n");

    sub.notifyStrategy = ALIMNS_TOPIC_EXPONENTIAL_DECAY_RETRY;
    [topic setSubscriptionAttributes:sub];

    AliMNSSubscription* sub2 = [topic getSubscriptionAttributes:testSubName2];
    XCTAssert(sub2 != nil, @"expected get subscripition attributes successfully.\n");
    XCTAssert([sub2.filterTag isEqual:sub.filterTag], @"expected get filterflag correctly.\n");
    XCTAssert([sub2.endpoint isEqual:sub.endpoint], @"expected get endpoint correctly.\n");
    XCTAssert(sub2.notifyContentFormat == sub.notifyContentFormat, @"expected get notifyContentFormat correctly.\n");
    XCTAssert(sub2.notifyStrategy == sub.notifyStrategy, @"expected get notifyStrategy correctly.\n");
}

-(void) testGetSubscriptionAttributes
{
    AliMNSTopic* topic = [account getTopic:testTopicName3];
    AliMNSSubscription* sub = [[AliMNSSubscription alloc] initWithTopicName: testTopicName3 withSubscriptionName:testSubName2];

    sub.filterTag = @"testFlag";
    sub.endpoint = @"https://test2.com";
    sub.notifyStrategy = ALIMNS_TOPIC_BACKOFF_RETRY;
    sub.notifyContentFormat = ALIMNS_TOPIC_SIMPLIFIED;

    NSString* suburl = [topic subscribe:sub];
    XCTAssert(suburl != nil, @"expected subscribe successfully.\n");

    AliMNSSubscription* sub2 = [topic getSubscriptionAttributes:testSubName2];
    XCTAssert(sub2 != nil, @"expected get subscripition attributes successfully.\n");
    XCTAssert([sub2.filterTag isEqual:sub.filterTag], @"expected get filterflag correctly.\n");
    XCTAssert([sub2.endpoint isEqual:sub.endpoint], @"expected get endpoint correctly.\n");
    XCTAssert(sub2.notifyContentFormat == sub.notifyContentFormat, @"expected get notifyContentFormat correctly.\n");
    XCTAssert(sub2.notifyStrategy == sub.notifyStrategy, @"expected get notifyStrategy correctly.\n");
}

-(void) testAsyncSendMessage
{
    AliMNSQueue* queue = [account getQueue:testQueueName3];
    AliMNSMessage* message = [AliMNSMessage new];
    message.messageBody = @"test message";

    [queue asyncSendMessage: message withSuccessHandler:^(AliMNSMessage *msg) {
        XCTAssert(msg != nil, @"return message should not be nil.");
        XCTAssert(msg.messageId != nil, @"message id should not be nil");
        XCTAssert(msg.messageBodyMd5 != nil, @"message body md5 should not be nil");
        NSLog(@"Callback:reqId=%@, msgId=%@, msgMd5=%@", msg.requestId, msg.messageId, msg.messageBodyMd5);
     }
    withFailHandler:^(AliMNSError* err)
     {
        XCTAssert(false, @"expect send message successfully.");
     }
    ];
    
    [NSThread sleepForTimeInterval:3.0f]; //wait enough time for async callback.
}

-(void) testAsyncSendMessageFail
{
    AliMNSQueue* queue = [accountWithWrongAK getQueue:testQueueName3];
    AliMNSMessage* message = [AliMNSMessage new];
    message.messageBody = @"test message";
    
    [queue asyncSendMessage: message withSuccessHandler:^(AliMNSMessage *msg) {
        XCTAssert(false, @"expect send message fail.");
     }
     withFailHandler:^(AliMNSError* err)
     {
         XCTAssert(true, @"expect send message fail.");
         XCTAssert(err.errorCode != nil, @"expect errorCode not nil.");
         XCTAssert(err.errorMessage != nil, @"expect errorMessage not nil.");
         XCTAssert(err.requestId != nil, @"expect requestId not nil.");
         XCTAssert(err.hostId != nil, @"expect hostId not nil.");
         NSLog(@"errorCode=%@, errorMessage=%@, requestId=%@, remoteHostId=%@", err.errorCode, err.errorMessage, err.requestId, err.hostId);
     }
     ];

    [NSThread sleepForTimeInterval:3.0f]; //wait enough time for async callback.
}

-(void) testAsyncBatchSendMessage
{
    AliMNSQueue* queue = [account getQueue:testQueueName3];

    NSMutableArray* msgList = [NSMutableArray new];
    NSInteger numOfMsg = 6;
    for (NSInteger i = 0; i < numOfMsg; i++)
    {
        AliMNSMessage* message = [AliMNSMessage new];
        message.messageBody = @"test message";
        [msgList addObject:message];
    }

    [queue asyncBatchSendMessage: msgList withSuccessHandler:^(NSArray *msgList) {

        XCTAssert(msgList != nil, @"return message list should not be nil.");
        XCTAssert(msgList.count == numOfMsg, @"return message should not be nil.");
        AliMNSMessage* msg = (AliMNSMessage*)msgList[0];
        XCTAssert(msg.messageId != nil, @"message id should not be nil");
        XCTAssert(msg.messageBodyMd5 != nil, @"message body md5 should not be nil");
        NSLog(@"Callback:reqId=%@, msgId=%@, msgMd5=%@", msg.requestId, msg.messageId, msg.messageBodyMd5);
    }
    withFailHandler:^(AliMNSError* err)
    {
         XCTAssert(false, @"expect batch send message successfully.");
    }
    ];
    
    [NSThread sleepForTimeInterval:3.0f]; //wait enough time for async callback.
}

-(void) testAsyncBatchSendMessageFail
{
    AliMNSQueue* queue = [accountWithWrongAK getQueue:testQueueName3];
    
    NSMutableArray* msgList = [NSMutableArray new];
    NSInteger numOfMsg = 6;
    for (NSInteger i = 0; i < numOfMsg; i++)
    {
        AliMNSMessage* message = [AliMNSMessage new];
        message.messageBody = @"test message";
        [msgList addObject:message];
    }

    [queue asyncBatchSendMessage: msgList withSuccessHandler:^(NSArray *msgList) {
        XCTAssert(false, @"expect batch send message fail.");
    }
            withFailHandler:^(AliMNSError* err)
     {
         XCTAssert(true, @"expect batch send message fail.");
         XCTAssert(err.errorCode != nil, @"expect errorCode not nil.");
         XCTAssert(err.errorMessage != nil, @"expect errorMessage not nil.");
         XCTAssert(err.requestId != nil, @"expect requestId not nil.");
         XCTAssert(err.hostId != nil, @"expect hostId not nil.");
         NSLog(@"errorCode=%@, errorMessage=%@, requestId=%@, remoteHostId=%@", err.errorCode, err.errorMessage, err.requestId, err.hostId);
     }
     ];

    [NSThread sleepForTimeInterval:3.0f]; //wait enough time for async callback.
}

-(void) testAsyncPeekMessage
{
    AliMNSQueue* queue = [account getQueue:testQueueName3];
    
    [queue asyncPeekMessageWithSuccessHandler:^(AliMNSMessage *msg) {

        XCTAssert(msg != nil, @"return message should not be nil.");
        XCTAssert(msg.messageId != nil, @"message id should not be nil");
        XCTAssert(msg.messageBodyMd5 != nil, @"message body md5 should not be nil");
        NSLog(@"Callback:reqId=%@, msgId=%@, msgMd5=%@", msg.requestId, msg.messageId, msg.messageBodyMd5);
    }
            withFailHandler:^(AliMNSError* err)
     {
         XCTAssert(false, @"expect peek message successfully.");
     }
     ];
    
    [NSThread sleepForTimeInterval:3.0f]; //wait enough time for async callback.
}

-(void) testAsyncPeekMessageFail
{
    AliMNSQueue* queue = [accountWithWrongAK getQueue:testQueueName3];
    
    [queue asyncPeekMessageWithSuccessHandler:^(AliMNSMessage *msg)  {
        XCTAssert(false, @"expect peek message fail.");
    }
               withFailHandler:^(AliMNSError* err)
     {
         XCTAssert(true, @"expect peek message fail.");
         XCTAssert(err.errorCode != nil, @"expect errorCode not nil.");
         XCTAssert(err.errorMessage != nil, @"expect errorMessage not nil.");
         XCTAssert(err.requestId != nil, @"expect requestId not nil.");
         XCTAssert(err.hostId != nil, @"expect hostId not nil.");
         NSLog(@"errorCode=%@, errorMessage=%@, requestId=%@, remoteHostId=%@", err.errorCode, err.errorMessage, err.requestId, err.hostId);
     }
     ];

    [NSThread sleepForTimeInterval:3.0f]; //wait enough time for async callback.
}

-(void) testAsyncBatchPeekMessage
{
    AliMNSQueue* queue = [account getQueue:testQueueName3];

    NSInteger numOfMsg = 3;
    [queue asyncBatchPeekMessage:numOfMsg withSuccessHandler:^(NSArray *msgList) {

        XCTAssert(msgList != nil, @"return message list should not be nil.");
        XCTAssert(msgList.count == numOfMsg, @"return message should not be nil.");
        AliMNSMessage* msg = (AliMNSMessage*)msgList[0];
        XCTAssert(msg.messageId != nil, @"message id should not be nil");
        XCTAssert(msg.messageBodyMd5 != nil, @"message body md5 should not be nil");
        NSLog(@"Callback:reqId=%@, msgId=%@, msgMd5=%@", msg.requestId, msg.messageId, msg.messageBodyMd5);
    }
                    withFailHandler:^(AliMNSError* err)
     {
         XCTAssert(false, @"expect batch peek message successfully.");
     }
     ];

    [NSThread sleepForTimeInterval:3.0f]; //wait enough time for async callback.
}

-(void) testAsyncBatchPeekMessageFail
{
    AliMNSQueue* queue = [accountWithWrongAK getQueue:testQueueName3];

    NSInteger numOfMsg = 3;
    [queue asyncBatchPeekMessage: numOfMsg withSuccessHandler:^(NSArray *msgList) {
        XCTAssert(false, @"expect batch peek message fail.");
    }
                    withFailHandler:^(AliMNSError* err)
     {
         XCTAssert(true, @"expect batch peek message fail.");
         XCTAssert(err.errorCode != nil, @"expect errorCode not nil.");
         XCTAssert(err.errorMessage != nil, @"expect errorMessage not nil.");
         XCTAssert(err.requestId != nil, @"expect requestId not nil.");
         XCTAssert(err.hostId != nil, @"expect hostId not nil.");
         NSLog(@"errorCode=%@, errorMessage=%@, requestId=%@, remoteHostId=%@", err.errorCode, err.errorMessage, err.requestId, err.hostId);
     }
     ];

    [NSThread sleepForTimeInterval:3.0f]; //wait enough time for async callback.
}

-(void) testAsyncReceiveMessage
{
    AliMNSQueue* queue = [account getQueue:testQueueName3];

    [queue asyncReceiveMessage: 3 withSuccessHandler:^(AliMNSMessage *msg) {
        
        XCTAssert(msg != nil, @"return message should not be nil.");
        XCTAssert(msg.messageId != nil, @"message id should not be nil");
        XCTAssert(msg.messageBodyMd5 != nil, @"message body md5 should not be nil");
        NSLog(@"Callback:reqId=%@, msgId=%@, msgMd5=%@", msg.requestId, msg.messageId, msg.messageBodyMd5);
    }
    withFailHandler:^(AliMNSError* err)
    {
         XCTAssert(false, @"expect receive message successfully.");
    }
    ];

    [NSThread sleepForTimeInterval:3.0f]; //wait enough time for async callback.
}

-(void) testAsyncReceiveMessageFail
{
    AliMNSQueue* queue = [accountWithWrongAK getQueue:testQueueName3];

    [queue asyncReceiveMessage: 3 withSuccessHandler:^(AliMNSMessage *msg)  {
        XCTAssert(false, @"expect receive message fail.");
    }
            withFailHandler:^(AliMNSError* err)
     {
         XCTAssert(true, @"expect receive message fail.");
         XCTAssert(err.errorCode != nil, @"expect errorCode not nil.");
         XCTAssert(err.errorMessage != nil, @"expect errorMessage not nil.");
         XCTAssert(err.requestId != nil, @"expect requestId not nil.");
         XCTAssert(err.hostId != nil, @"expect hostId not nil.");
         NSLog(@"errorCode=%@, errorMessage=%@, requestId=%@, remoteHostId=%@", err.errorCode, err.errorMessage, err.requestId, err.hostId);
     }
     ];
    
    [NSThread sleepForTimeInterval:3.0f]; //wait enough time for async callback.
}

-(void) testAsyncBatchReceiveMessage
{
    AliMNSQueue* queue = [account getQueue:testQueueName3];

    NSInteger numOfMsg = 3;
    [queue asyncBatchReceiveMessage: 3 withBatchSize: numOfMsg withSuccessHandler:^(NSArray *msgList) {
        
        XCTAssert(msgList != nil, @"return message list should not be nil.");
        XCTAssert(msgList.count == numOfMsg, @"return message should not be nil.");
        AliMNSMessage* msg = (AliMNSMessage*)msgList[0];
        XCTAssert(msg.messageId != nil, @"message id should not be nil");
        XCTAssert(msg.messageBodyMd5 != nil, @"message body md5 should not be nil");
        NSLog(@"Callback:reqId=%@, msgId=%@, msgMd5=%@", msg.requestId, msg.messageId, msg.messageBodyMd5);
    }
    withFailHandler:^(AliMNSError* err)
    {
        XCTAssert(false, @"expect batch receive message successfully.");
    }
    ];

    [NSThread sleepForTimeInterval:3.0f]; //wait enough time for async callback.
}

-(void) testAsyncBatchReceiveMessageFail
{
    AliMNSQueue* queue = [accountWithWrongAK getQueue:testQueueName3];

    NSInteger numOfMsg = 3;
    [queue asyncBatchReceiveMessage: 3 withBatchSize: numOfMsg withSuccessHandler:^(NSArray *msgList) {
        XCTAssert(false, @"expect batch receive message fail.");
    }
    withFailHandler:^(AliMNSError* err)
    {
         XCTAssert(true, @"expect batch receive message fail.");
         XCTAssert(err.errorCode != nil, @"expect errorCode not nil.");
         XCTAssert(err.errorMessage != nil, @"expect errorMessage not nil.");
         XCTAssert(err.requestId != nil, @"expect requestId not nil.");
         XCTAssert(err.hostId != nil, @"expect hostId not nil.");
         NSLog(@"errorCode=%@, errorMessage=%@, requestId=%@, remoteHostId=%@", err.errorCode, err.errorMessage, err.requestId, err.hostId);
    }
    ];

    [NSThread sleepForTimeInterval:3.0f]; //wait enough time for async callback.
}

-(void) testAsyncDeleteMessage
{
    AliMNSQueue* queue = [account getQueue:testQueueName3];
    AliMNSMessage* msg = [queue receiveMessage:0];
    [queue asyncDeleteMessage:msg.receiptHandle withSuccessHandler:^{
        XCTAssert(true, @"expect delete message successfully.");
    } withFailHandler:^(AliMNSError *err) {
        XCTAssert(false, @"expect delete message successfully.");
        NSLog(@"errorCode=%@, errorMessage=%@, requestId=%@, remoteHostId=%@", err.errorCode, err.errorMessage, err.requestId, err.hostId);
    }];
    [NSThread sleepForTimeInterval:3.0f]; //wait enough time for async callback.
}

-(void) testAsyncDeleteMessageFail
{
    AliMNSQueue* queue = [account getQueue:testQueueName3];
    AliMNSMessage* msg = [queue receiveMessage:0];

    AliMNSQueue* queue2 = [accountWithWrongAK getQueue:testQueueName3];
    [queue2 asyncDeleteMessage:msg.receiptHandle withSuccessHandler:^{
        XCTAssert(false, @"expect delete message fail.");
    } withFailHandler:^(AliMNSError *err) {
        XCTAssert(true, @"expect delete message fail.");
    }];
    [NSThread sleepForTimeInterval:3.0f]; //wait enough time for async callback.
}

-(void) testAsyncBatchDeleteMessage
{
    AliMNSQueue* queue = [account getQueue:testQueueName3];
    NSMutableArray* msgList = [NSMutableArray new];
    NSInteger numOfMsg = 3;
    for(NSInteger i = 0; i < numOfMsg; i++)
    {
        AliMNSMessage* msg = [queue receiveMessage:0];
        [msgList addObject:msg];
    }

    [queue asyncBatchSendMessage:msgList withSuccessHandler:^(NSArray *msgList) {
        XCTAssert(true, @"expect batch delete message successfully.");
    } withFailHandler:^(AliMNSError *err) {
        XCTAssert(false, @"expect batch delete message successfully.");
    }];
    [NSThread sleepForTimeInterval:3.0f]; //wait enough time for async callback.
}

-(void) testAsyncBatchDeleteMessageFail
{
    AliMNSQueue* queue = [account getQueue:testQueueName3];
    NSMutableArray* msgList = [NSMutableArray new];
    NSInteger numOfMsg = 3;
    for(NSInteger i = 0; i < numOfMsg; i++)
    {
        AliMNSMessage* msg = [queue receiveMessage:0];
        [msgList addObject:msg];
    }

    AliMNSQueue* queue2 = [accountWithWrongAK getQueue:testQueueName3];
    [queue2 asyncBatchSendMessage:msgList withSuccessHandler:^(NSArray *msgList) {
        XCTAssert(false, @"expect batch delete message successfully.");
    } withFailHandler:^(AliMNSError *err) {
        XCTAssert(true, @"expect batch delete message successfully.");
    }];
    [NSThread sleepForTimeInterval:3.0f]; //wait enough time for async callback.
}

-(void) testAsyncPublishMessage
{
    AliMNSTopic* topic = [account getTopic:testTopicName3];
    AliMNSTopicMessage* tMsg = [AliMNSTopicMessage new];
    tMsg.messageBody = @"test topic message";
    tMsg.messageTag = @"Ali";

    [topic asyncPublishMessage: tMsg withSuccessHandler:^(AliMNSTopicMessage *msg) {
        
        XCTAssert(msg != nil, @"return message should not be nil.");
        XCTAssert(msg.messageId != nil, @"message id should not be nil");
        XCTAssert(msg.messageBodyMd5 != nil, @"message body md5 should not be nil");
        NSLog(@"Callback:reqId=%@, msgId=%@, msgMd5=%@", msg.requestId, msg.messageId, msg.messageBodyMd5);
    }
    withFailHandler:^(AliMNSError* err){
        XCTAssert(false, @"expect publish message successfully.");
    }];
    [NSThread sleepForTimeInterval:3.0f]; //wait enough time for async callback.
}

-(void) testAsyncPublishMessageFail
{
    AliMNSTopic* topic = [accountWithWrongAK getTopic:testTopicName3];
    AliMNSTopicMessage* tMsg = [AliMNSTopicMessage new];
    tMsg.messageBody = @"test topic message";
    tMsg.messageTag = @"Ali";

    [topic asyncPublishMessage: tMsg withSuccessHandler:^(AliMNSTopicMessage *msg) {
        XCTAssert(false, @"expect publish message fail.");
    }withFailHandler:^(AliMNSError* err){
         XCTAssert(true, @"expect publish message fail.");
         XCTAssert(err.errorCode != nil, @"expect errorCode not nil.");
         XCTAssert(err.errorMessage != nil, @"expect errorMessage not nil.");
         XCTAssert(err.requestId != nil, @"expect requestId not nil.");
         XCTAssert(err.hostId != nil, @"expect hostId not nil.");
         NSLog(@"errorCode=%@, errorMessage=%@, requestId=%@, remoteHostId=%@", err.errorCode, err.errorMessage, err.requestId, err.hostId);
    }];

    [NSThread sleepForTimeInterval:3.0f]; //wait enough time for async callback.
}

@end
